@@ -129,7 +129,9 @@ simpleKey <-
              
 
 
-
+componentName <- function(name, x, y) {
+    trellis.grobname(paste(name, x, y, sep="."), type="key")
+}
 
 
 draw.key <- function(key, draw = FALSE, vp = NULL, ...)
@@ -512,19 +514,24 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                         just = if (is.null(key$just)) "center" else key$just)
 
         ## OK, layout set up, now to draw the key - no
-        key.gf <- frameGrob(layout = key.layout, vp = vp)
+        key.gf <- frameGrob(layout = key.layout, vp = vp,
+                            name = trellis.grobname("frame", type="key"))
         if (!key$transparent)
             key.gf <-
                 placeGrob(key.gf,
                           rectGrob(gp =
                                    gpar(fill = key$background,
                                         alpha = key$alpha.background,
-                                        col = key$border)),
+                                        col = key$border),
+                                   name = trellis.grobname("background",
+                                     type="key")),
                           row = NULL, col = NULL)
         else
             key.gf <-
                 placeGrob(key.gf,
-                          rectGrob(gp=gpar(col=key$border)),
+                          rectGrob(gp=gpar(col=key$border),
+                                   name = trellis.grobname("background",
+                                     type="key")),
                           row = NULL, col = NULL)
 
         ## Title (FIXME: allow color, font, alpha-transparency here? 
@@ -532,7 +539,10 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
             key.gf <-
                 placeGrob(key.gf, 
                           textGrob(label = key$title,
-                                   gp = gpar(cex = key$cex.title, lineheight = key$lineheight)),
+                                   gp = gpar(cex = key$cex.title,
+                                     lineheight = key$lineheight),
+                                   name = trellis.grobname("title",
+                                     type="key")),
                           row = 1, col = NULL)
         
         for (i in 1:number.of.components)
@@ -545,6 +555,8 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                     (number.of.components*3 + 1) + i*3 - 1
                 yy <- j %% rows.per.block + 1
                 if (yy == 1) yy <- rows.per.block + 1
+                componentx <- (colblck - 1)*(number.of.components) + i 
+                componenty <- (j - 1) %% rows.per.block + 1
                 if (cur$type == "text")
                 {
                     key.gf <-
@@ -566,7 +578,9 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                                                 lineheight = cur$pars$lineheight[j],
                                                 fontfamily = cur$pars$fontfamily[j],
                                                 fontface = chooseFace(cur$pars$fontface[j], cur$pars$font[j]),
-                                                cex = cur$pars$cex[j])),
+                                                cex = cur$pars$cex[j]),
+                                           name = componentName("text",
+                                             componentx, componenty)),
                                   row = yy, col = xx)
                 }
                 else if (cur$type == "rectangles")
@@ -579,7 +593,9 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                                            ## centred, unlike S-PLUS, due to aesthetic reasons !
                                            gp = gpar(alpha = cur$pars$alpha[j],
                                                      fill = cur$pars$col[j],
-                                                     col = cur$pars$border[j])),
+                                                     col = cur$pars$border[j]),
+                                           name = componentName("rect",
+                                             componentx, componenty)),
                                   row = yy, col = xx)
                     ## FIXME: Need to make changes to support angle/density
                 }
@@ -607,7 +623,9 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                                                 gpar(col = cur$pars$col[j],
                                                      alpha = cur$pars$alpha[j],
                                                      lty = cur$pars$lty[j],
-                                                     lwd = cur$pars$lwd[j])),
+                                                     lwd = cur$pars$lwd[j]),
+                                                name = componentName("lines",
+                                                  componentx, componenty)),
                                       row = yy, col = xx)
                     }
                     else if (cur$pars$type[j] == "p")
@@ -623,7 +641,9 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                                                       fontfamily = cur$pars$fontfamily[j],
                                                       fontface = chooseFace(cur$pars$fontface[j], cur$pars$font[j]),
                                                       fontsize = fontsize.points),
-                                                 pch = cur$pars$pch[j]),
+                                                 pch = cur$pars$pch[j],
+                                                 name = componentName("points",
+                                                   componentx, componenty)),
                                       row = yy, col = xx)
                     }
                     else # if (cur$pars$type[j] == "b" or "o") -- not differentiating
@@ -648,7 +668,9 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                                                 gpar(col = cur$pars$col[j],
                                                      alpha = cur$pars$alpha[j],
                                                      lty = cur$pars$lty[j],
-                                                     lwd = cur$pars$lwd[j])),
+                                                     lwd = cur$pars$lwd[j]),
+                                                name = componentName("lines",
+                                                  componentx, componenty)),
                                       row = yy, col = xx)
                         if (key$divide > 1)
                         {
@@ -664,7 +686,9 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                                                           fontfamily = cur$pars$fontfamily[j],
                                                           fontface = chooseFace(cur$pars$fontface[j], cur$pars$font[j]),
                                                           fontsize = fontsize.points),
-                                                     pch = cur$pars$pch[j]),
+                                                     pch = cur$pars$pch[j],
+                                                     name = componentName("points",
+                                                       componentx, componenty)),
                                           row = yy, col = xx)
                         }
                         else if (key$divide == 1)
@@ -681,7 +705,9 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                                                           fontfamily = cur$pars$fontfamily[j],
                                                           fontface = chooseFace(cur$pars$fontface[j], cur$pars$font[j]),
                                                           fontsize = fontsize.points),
-                                                     pch = cur$pars$pch[j]),
+                                                     pch = cur$pars$pch[j],
+                                                     name = componentName("points",
+                                                       componentx, componenty)),
                                           row = yy, col = xx)
                         }
                     }
@@ -700,7 +726,9 @@ draw.key <- function(key, draw = FALSE, vp = NULL, ...)
                                                   fontfamily = cur$pars$fontfamily[j],
                                                   fontface = chooseFace(cur$pars$fontface[j], cur$pars$font[j]),
                                                   fontsize = fontsize.points),
-                                             pch = cur$pars$pch[j]),
+                                             pch = cur$pars$pch[j],
+                                             name = componentName("points",
+                                               componentx, componenty)),
                                   row = yy, col = xx)
                 }
             }
@@ -728,11 +756,14 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                  alpha = regions$alpha,
                  at,
                  tick.number = 7,
+                 tck = 1,
                  width = 2,
                  height = 1,
                  space = "right",
                  raster = FALSE,
                  interpolate = FALSE,
+                 axis.line = list(),
+                 axis.text = list(),
                  ...)
         {
             regions <- trellis.par.get("regions")
@@ -740,19 +771,22 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                  alpha = alpha,
                  at = at,
                  tick.number = tick.number,
+                 tck = tck,
                  width = width,
                  height = height,
                  space = space,
                  raster = raster,
                  interpolate = interpolate,
+                 axis.line = axis.line,
+                 axis.text = axis.text,
                  ...)
         }
 
-    axis.line <- trellis.par.get("axis.line")
-    axis.text <- trellis.par.get("axis.text")
-
     key <- do.call(process.key, key)
 
+    axis.line <- updateList(trellis.par.get("axis.line"), key$axis.line)
+    axis.text <- updateList(trellis.par.get("axis.text"), key$axis.text)
+
 ## FIXME: delete later
 #str(key)
     ## made FALSE later if labels explicitly specified
@@ -763,7 +797,7 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
     ## the breakpoints of the rectangles, and the other is key$lab$at
     ## (optional) which is the positions of the ticks. We will use the
     ## 'at' variable for the latter, 'atrange' for the range of the
-    ## former, and keyat explicitly when needed
+    ## former, and key$at explicitly when needed
 
 
 
@@ -805,6 +839,23 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
     lineheight <- axis.text$lineheight
     rot <- 0
 
+    ## The following code assumes names key$lab and key$lab$lab (which
+    ## may have been used in user code), whereas documentation says
+    ## key$labels and key$labels$labels.  To make both work without
+    ## 'partial matching' warnings, we rename key$labels to key$lab
+    ## etc.
+
+    if (!is.null(key[["labels"]])) 
+    {
+        key[["lab"]] <- key[["labels"]]
+        key[["labels"]] <- NULL
+        if (is.list(key[["lab"]]) && !is.null(key[["lab"]][["labels"]])) 
+        {
+            key[["lab"]][["lab"]] <- key[["lab"]][["labels"]]
+            key[["lab"]][["labels"]] <- NULL
+        }
+    }
+    
     if (is.null(key$lab))
     {
         at <- lpretty(atrange, key$tick.number)
@@ -837,29 +888,33 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
     else stop("malformed colorkey")
 
     labscat <- at
+    do.labels <- (length(labscat) > 0)
 
-    if (key$space == "right") {
-
+    if (key$space == "right")
+    {
         labelsGrob <-
-            textGrob(label = labels,
-                     x = rep(0, length(labscat)),
-                     y = labscat,
-                     vp = viewport(yscale = atrange),
-                     default.units = "native",
-                     check.overlap = check.overlap,
-                     just = if (rot == -90) c("center", "bottom") else c("left", "center"),
-                     rot = rot,
-                     gp =
-                     gpar(col = col,
-                          cex = cex,
-                          fontfamily = fontfamily,
-                          fontface = chooseFace(fontface, font),
-                          lineheight = lineheight))
+            if (do.labels)
+                textGrob(label = labels,
+                         x = rep(0, length(labscat)),
+                         y = labscat,
+                         vp = viewport(yscale = atrange),
+                         default.units = "native",
+                         check.overlap = check.overlap,
+                         just = if (rot == -90) c("center", "bottom") else c("left", "center"),
+                         rot = rot,
+                         name = trellis.grobname("labels", type="colorkey"),
+                         gp =
+                         gpar(col = col,
+                              cex = cex,
+                              fontfamily = fontfamily,
+                              fontface = chooseFace(fontface, font),
+                              lineheight = lineheight))
+            else nullGrob()
 
         heights.x <- c((1 - key$height) / 2, key$height, (1 - key$height) / 2)
         heights.units <- rep("null", 3)
 
-        widths.x <- c(.6 * key$width, .6, 1)
+        widths.x <- c(0.6 * key$width, do.labels * (0.3 + key$tck * 0.3), do.labels * 1)
         widths.units <- c("lines", "lines", "grobwidth")
         widths.data <- list(NULL, NULL, labelsGrob)
         
@@ -869,14 +924,17 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                         widths = unit(widths.x, widths.units, data = widths.data),
                         respect = TRUE)
 
-        key.gf <- frameGrob(layout = key.layout, vp = vp)
-
+        key.gf <- frameGrob(layout = key.layout, vp = vp,
+                            name = trellis.grobname("frame",
+                              type="colorkey"))
         if (key$raster)
         {
             key.gf <- placeGrob(key.gf,
                                 rasterGrob(matrix(rev(key$col), ncol = 1),
                                            width = 1, height = 1,
                                            vp = viewport(clip = "on"),
+                                           name = trellis.grobname("raster",
+                                                                   type="colorkey"),
                                            interpolate = key$interpolate),
                                 row = 2, col = 1)
         }
@@ -888,66 +946,72 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                                          default.units = "native",
                                          vp = viewport(yscale = atrange),
                                          height = recdim, 
+                                         name = trellis.grobname("image",
+                                                                 type="colorkey"),
                                          gp =
                                          gpar(fill = key$col,
                                               col = "transparent",
                                               alpha = key$alpha)),
                                 row = 2, col = 1)
         }
-        
         key.gf <- placeGrob(frame = key.gf, 
-                            rectGrob(gp =
+                            rectGrob(name = trellis.grobname("border",
+                                       type="colorkey"),
+                                     gp =
                                      gpar(col = axis.line$col,
                                           lty = axis.line$lty,
                                           lwd = axis.line$lwd,
                                           alpha = axis.line$alpha,
                                           fill = "transparent")),
                             row = 2, col = 1)
-
-
-        key.gf <- placeGrob(frame = key.gf, 
-                            segmentsGrob(x0 = rep(0, length(labscat)),
-                                         y0 = labscat,
-                                         x1 = rep(.4, length(labscat)),
-                                         y1 = labscat,
-                                         vp = viewport(yscale = atrange),
-                                         default.units = "native",
-                                         gp =
-                                         gpar(col = axis.line$col,
-                                              lty = axis.line$lty,
-                                              lwd = axis.line$lwd)),
-                            row = 2, col = 2)
-        
-        key.gf <- placeGrob(key.gf,
-                            labelsGrob, 
-                            row = 2, col = 3)
-#        key.gf <- placeGrob(key.gf,
-#                            rectGrob(), 
-#                            row = 2, col = 3)
-
+        if (do.labels)
+        {
+            if (key$tck != 0)
+                key.gf <- placeGrob(frame = key.gf, 
+                                    segmentsGrob(x0 = rep(0, length(labscat)),
+                                                 y0 = labscat,
+                                                 x1 = rep(key$tck / (1 + key$tck), length(labscat)),
+                                                 y1 = labscat,
+                                                 vp = viewport(yscale = atrange),
+                                                 default.units = "native",
+                                                 name = trellis.grobname("ticks",
+                                                                         type="colorkey"),
+                                                 gp =
+                                                 gpar(col = axis.line$col,
+                                                      lty = axis.line$lty,
+                                                      lwd = axis.line$lwd)),
+                                    row = 2, col = 2)
+            key.gf <- placeGrob(key.gf,
+                                labelsGrob, 
+                                row = 2, col = 3)
+        }
     }
-    else if (key$space == "left") {
-
+    else if (key$space == "left")
+    {
         labelsGrob <-
-            textGrob(label = labels,
-                     x = rep(1, length(labscat)),
-                     y = labscat,
-                     vp = viewport(yscale = atrange),
-                     default.units = "native",
-                     check.overlap = check.overlap,
-                     just = if (rot == 90) c("center", "bottom") else c("right", "center"),
-                     rot = rot,
-                     gp =
-                     gpar(col = col,
-                          cex = cex,
-                          fontfamily = fontfamily,
-                          fontface = chooseFace(fontface, font),
-                          lineheight = lineheight))
+            if (do.labels)
+                textGrob(label = labels,
+                         x = rep(1, length(labscat)),
+                         y = labscat,
+                         vp = viewport(yscale = atrange),
+                         default.units = "native",
+                         check.overlap = check.overlap,
+                         just = if (rot == 90) c("center", "bottom") else c("right", "center"),
+                         rot = rot,
+                         name = trellis.grobname("labels", type="colorkey"),
+                         gp =
+                         gpar(col = col,
+                              cex = cex,
+                              fontfamily = fontfamily,
+                              fontface = chooseFace(fontface, font),
+                              lineheight = lineheight))
+            else nullGrob()
 
         heights.x <- c((1 - key$height) / 2, key$height, (1 - key$height) / 2)
         heights.units <- rep("null", 3)
 
-        widths.x <- c(1, .6, .6 * key$width)
+
+        widths.x <- c(do.labels * 1, do.labels * (0.3 + key$tck * 0.3), 0.6 * key$width)
         widths.units <- c("grobwidth", "lines", "lines")
         widths.data <- list(labelsGrob, NULL, NULL)
         
@@ -956,15 +1020,18 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                         heights = unit(heights.x, heights.units),
                         widths = unit(widths.x, widths.units, data = widths.data),
                         respect = TRUE)
-        
-        key.gf <- frameGrob(layout = key.layout, vp = vp)
 
+        key.gf <- frameGrob(layout = key.layout, vp = vp,
+                            name = trellis.grobname("frame",
+                              type="colorkey"))
         if (key$raster)
         {
             key.gf <- placeGrob(key.gf,
                                 rasterGrob(matrix(rev(key$col), ncol = 1),
                                            width = 1, height = 1,
                                            vp = viewport(clip = "on"),
+                                           name = trellis.grobname("raster",
+                                             type="colorkey"),
                                            interpolate = key$interpolate),
                                 row = 2, col = 3)
         }
@@ -976,63 +1043,71 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                                          default.units = "native",
                                          vp = viewport(yscale = atrange),
                                          height = recdim, 
+                                         name = trellis.grobname("image",
+                                           type="colorkey"),
                                          gp =
                                          gpar(fill = key$col,
                                               col = "transparent",
                                               alpha = key$alpha)),
                                 row = 2, col = 3)
         }
-        
         key.gf <- placeGrob(frame = key.gf, 
-                            rectGrob(gp =
+                            rectGrob(name = trellis.grobname("border",
+                                             type="colorkey"),
+                                     gp =
                                      gpar(col = axis.line$col,
                                           lty = axis.line$lty,
                                           lwd = axis.line$lwd,
                                           alpha = axis.line$alpha,
                                           fill = "transparent")),
                             row = 2, col = 3)
-
-
-        key.gf <- placeGrob(frame = key.gf, 
-                            segmentsGrob(x0 = rep(1, length(labscat)),
-                                         y0 = labscat,
-                                         x1 = rep(.6, length(labscat)),
-                                         y1 = labscat,
-                                         vp = viewport(yscale = atrange),
-                                         default.units = "native",
-                                         gp =
-                                         gpar(col = axis.line$col,
-                                              lty = axis.line$lty,
-                                              lwd = axis.line$lwd)),
-                            row = 2, col = 2)
-        
-        key.gf <- placeGrob(key.gf,
-                            labelsGrob, 
-                            row = 2, col = 1)
-
+        if (do.labels)
+        {
+            if (key$tck != 0)
+                key.gf <- placeGrob(frame = key.gf, 
+                                    segmentsGrob(x0 = rep(1, length(labscat)),
+                                                 y0 = labscat,
+                                                 x1 = rep(1 - key$tck / (1 + key$tck), length(labscat)),
+                                                 y1 = labscat,
+                                                 vp = viewport(yscale = atrange),
+                                                 default.units = "native",
+                                                 name = trellis.grobname("ticks",
+                                                                         type="colorkey"),
+                                                 gp =
+                                                 gpar(col = axis.line$col,
+                                                      lty = axis.line$lty,
+                                                      lwd = axis.line$lwd)),
+                                    row = 2, col = 2)
+            key.gf <- placeGrob(key.gf,
+                                labelsGrob, 
+                                row = 2, col = 1)
+        }
     }
-    else if (key$space == "top") {
-
+    else if (key$space == "top")
+    {
         labelsGrob <-
-            textGrob(label = labels,
-                     y = rep(0, length(labscat)),
-                     x = labscat,
-                     vp = viewport(xscale = atrange),
-                     default.units = "native",
-                     check.overlap = check.overlap,
-                     just = if (rot == 0) c("center","bottom") else c("left", "center"),
-                     rot = rot,
-                     gp =
-                     gpar(col = col,
-                          cex = cex,
-                          fontfamily = fontfamily,
-                          fontface = chooseFace(fontface, font),
-                          lineheight = lineheight))
+            if (do.labels)
+                textGrob(label = labels,
+                         y = rep(0, length(labscat)),
+                         x = labscat,
+                         vp = viewport(xscale = atrange),
+                         default.units = "native",
+                         check.overlap = check.overlap,
+                         just = if (rot == 0) c("center","bottom") else c("left", "center"),
+                         rot = rot,
+                         name = trellis.grobname("labels", type="colorkey"),
+                         gp =
+                         gpar(col = col,
+                              cex = cex,
+                              fontfamily = fontfamily,
+                              fontface = chooseFace(fontface, font),
+                              lineheight = lineheight))
+            else nullGrob()
 
         widths.x <- c((1 - key$height) / 2, key$height, (1 - key$height) / 2)
         widths.units <- rep("null", 3)
 
-        heights.x <- c(1, .6, .6 * key$width)
+        heights.x <- c(do.labels * 1, do.labels * (0.3 + key$tck * 0.3), 0.6 * key$width)
         heights.units <- c("grobheight", "lines", "lines")
         heights.data <- list(labelsGrob, NULL, NULL)
         
@@ -1042,14 +1117,17 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                         widths = unit(widths.x, widths.units),
                         respect = TRUE)
         
-        key.gf <- frameGrob(layout = key.layout, vp = vp)
-
+        key.gf <- frameGrob(layout = key.layout, vp = vp,
+                            name = trellis.grobname("frame",
+                              type="colorkey"))
         if (key$raster)
         {
             key.gf <- placeGrob(key.gf,
                                 rasterGrob(matrix(key$col, nrow = 1),
                                            width = 1, height = 1,
                                            vp = viewport(clip = "on"),
+                                           name = trellis.grobname("raster",
+                                             type="colorkey"),
                                            interpolate = key$interpolate),
                                 row = 3, col = 2)
         }
@@ -1061,64 +1139,71 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                                          default.units = "native",
                                          vp = viewport(xscale = atrange),
                                          width = recdim, 
+                                         name = trellis.grobname("image",
+                                           type="colorkey"),
                                          gp =
                                          gpar(fill = key$col,
                                               col = "transparent",
                                               alpha = key$alpha)),
                                 row = 3, col = 2)
         }
-        
         key.gf <- placeGrob(frame = key.gf, 
-                            rectGrob(gp =
+                            rectGrob(name = trellis.grobname("border",
+                                             type="colorkey"),
+                                     gp =
                                      gpar(col = axis.line$col,
                                           lty = axis.line$lty,
                                           lwd = axis.line$lwd,
                                           alpha = axis.line$alpha,
                                           fill = "transparent")),
                             row = 3, col = 2)
-
-
-        key.gf <- placeGrob(frame = key.gf, 
-                            segmentsGrob(y0 = rep(0, length(labscat)),
-                                         x0 = labscat,
-                                         y1 = rep(.4, length(labscat)),
-                                         x1 = labscat,
-                                         vp = viewport(xscale = atrange),
-                                         default.units = "native",
-                                         gp =
-                                         gpar(col = axis.line$col,
-                                              lty = axis.line$lty,
-                                              lwd = axis.line$lwd)),
-                            row = 2, col = 2)
-        
-        key.gf <- placeGrob(key.gf,
-                            labelsGrob, 
-                            row = 1, col = 2)
-
+        if (do.labels)
+        {
+            if (key$tck != 0)
+                key.gf <- placeGrob(frame = key.gf, 
+                                    segmentsGrob(y0 = rep(0, length(labscat)),
+                                                 x0 = labscat,
+                                                 y1 = rep(key$tck / (1 + key$tck), length(labscat)),
+                                                 x1 = labscat,
+                                                 vp = viewport(xscale = atrange),
+                                                 default.units = "native",
+                                                 name = trellis.grobname("ticks",
+                                                                         type="colorkey"),
+                                                 gp =
+                                                 gpar(col = axis.line$col,
+                                                      lty = axis.line$lty,
+                                                      lwd = axis.line$lwd)),
+                                    row = 2, col = 2)
+            key.gf <- placeGrob(key.gf,
+                                labelsGrob, 
+                                row = 1, col = 2)
+        }
     }
-    else if (key$space == "bottom") {
-
-
+    else if (key$space == "bottom")
+    {
         labelsGrob <-
-            textGrob(label = labels,
-                     y = rep(1, length(labscat)),
-                     x = labscat,
-                     vp = viewport(xscale = atrange),
-                     default.units = "native",
-                     check.overlap = check.overlap,
-                     just = if (rot == 0) c("center", "top") else c("right", "center"),
-                     rot = rot,
-                     gp =
-                     gpar(col = col,
-                          cex = cex,
-                          fontfamily = fontfamily,
-                          fontface = chooseFace(fontface, font),
-                          lineheight = lineheight))
+            if (do.labels)
+                textGrob(label = labels,
+                         y = rep(1, length(labscat)),
+                         x = labscat,
+                         vp = viewport(xscale = atrange),
+                         default.units = "native",
+                         check.overlap = check.overlap,
+                         just = if (rot == 0) c("center", "top") else c("right", "center"),
+                         rot = rot,
+                         name = trellis.grobname("labels", type="colorkey"),
+                         gp =
+                         gpar(col = col,
+                              cex = cex,
+                              fontfamily = fontfamily,
+                              fontface = chooseFace(fontface, font),
+                              lineheight = lineheight))
+            else nullGrob()
 
         widths.x <- c((1 - key$height) / 2, key$height, (1 - key$height) / 2)
         widths.units <- rep("null", 3)
 
-        heights.x <- c(.6 * key$width, .6, 1)
+        heights.x <- c(0.6 * key$width, do.labels * (0.3 + key$tck * 0.3), do.labels * 1)
         heights.units <- c("lines", "lines", "grobheight")
         heights.data <- list(NULL, NULL, labelsGrob)
         
@@ -1128,14 +1213,17 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                         widths = unit(widths.x, widths.units),
                         respect = TRUE)
         
-        key.gf <- frameGrob(layout = key.layout, vp = vp)
-
+        key.gf <- frameGrob(layout = key.layout, vp = vp,
+                            name = trellis.grobname("frame",
+                              type="colorkey"))
         if (key$raster)
         {
             key.gf <- placeGrob(key.gf,
                                 rasterGrob(matrix(key$col, nrow = 1),
                                            width = 1, height = 1,
                                            vp = viewport(clip = "on"),
+                                           name = trellis.grobname("raster",
+                                             type="colorkey"),
                                            interpolate = key$interpolate),
                                 row = 1, col = 2)
         }
@@ -1147,49 +1235,49 @@ draw.colorkey <- function(key, draw = FALSE, vp = NULL)
                                          default.units = "native",
                                          vp = viewport(xscale = atrange),
                                          width = recdim, 
+                                         name = trellis.grobname("image",
+                                           type="colorkey"),
                                          gp =
                                          gpar(fill = key$col,
                                               col = "transparent",
                                               alpha = key$alpha)),
                                 row = 1, col = 2)
         }
-        
         key.gf <- placeGrob(frame = key.gf, 
-                            rectGrob(gp =
+                            rectGrob(name = trellis.grobname("image",
+                                       type="colorkey"),
+                                     gp =
                                      gpar(col = axis.line$col,
                                           lty = axis.line$lty,
                                           lwd = axis.line$lwd,
                                           alpha = axis.line$alpha,
                                           fill = "transparent")),
                             row = 1, col = 2)
-
-        key.gf <- placeGrob(frame = key.gf, 
-                            segmentsGrob(y0 = rep(1, length(labscat)),
-                                         x0 = labscat,
-                                         y1 = rep(.6, length(labscat)),
-                                         x1 = labscat,
-                                         vp = viewport(xscale = atrange),
-                                         default.units = "native",
-                                         gp =
-                                         gpar(col = axis.line$col,
-                                              lty = axis.line$lty,
-                                              lwd = axis.line$lwd)),
-                            row = 2, col = 2)
-        
-        key.gf <- placeGrob(key.gf,
-                            labelsGrob, 
-                            row = 3, col = 2)
-
+        if (do.labels)
+        {
+            if (key$tck != 0)
+                key.gf <- placeGrob(frame = key.gf, 
+                                    segmentsGrob(y0 = rep(1, length(labscat)),
+                                                 x0 = labscat,
+                                                 y1 = rep(1 - key$tck / (1 + key$tck), length(labscat)),
+                                                 x1 = labscat,
+                                                 vp = viewport(xscale = atrange),
+                                                 default.units = "native",
+                                                 name = trellis.grobname("ticks",
+                                                                         type="colorkey"),
+                                                 gp =
+                                                 gpar(col = axis.line$col,
+                                                      lty = axis.line$lty,
+                                                      lwd = axis.line$lwd)),
+                                    row = 2, col = 2)
+            key.gf <- placeGrob(key.gf,
+                                labelsGrob, 
+                                row = 3, col = 2)
+        }
     }
-
-
     if (draw)
         grid.draw(key.gf)
-    
     key.gf
 }
 
 
-
-
-
